package jp.co.glossom.adfurikun

import io.flutter.plugin.common.MethodChannel
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunMovieError
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunMovieReward
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunMovieRewardListener
import jp.tjkapp.adfurikunsdk.moviereward.MovieRewardData

/**
 * 動画リワード
 */
internal class AdfurikunReward(var number: Int, val appId: String, val channel: MethodChannel) {

    private var reward: AdfurikunMovieReward? = null
    private var listener: AdfurikunMovieRewardListener? = null

    /**
     * 初期化
     */
    fun init() {
        if (AdfurikunPlugin.currentActivity != null) {
            reward = AdfurikunMovieReward(appId, AdfurikunPlugin.currentActivity).apply {
                setAdfurikunMovieRewardListener(createListener)
            }
        }
    }

    /**
     * トラッキングIDを設定
     * @param trackingId
     */
    fun setTrackingId(trackingId: HashMap<String, String>? = null) {
        reward?.setTrackingId(trackingId)
    }

    /**
     * 読み込み
     * @param timeout
     */
    fun load(timeout: Float) {
        if (timeout > 0) {
            reward?.loadWithTimeout(timeout)
        } else {
            reward?.load()
        }
    }

    /**
     * 再生
     * @param customParams
     */
    fun play(customParams: HashMap<String, String>? = null) {
        reward?.play(customParams)
    }

    /**
     * 準備判定
     * @param result
     */
    fun isPrepared(result: MethodChannel.Result) {
        result.success(reward?.isPrepared ?: false)
    }

    /**
     * 破棄
     */
    fun destroy() {
        reward?.onDestroy()
        reward = null
        listener = null
    }

    /**
     * リワード成功有無をアプリから通知
     * @param result
     */
    fun rewardCompleted(result: Boolean) {
        reward?.rewardCompleted(result)
    }

    /**
     * 在庫なし状態をアプリから通知
     */
    fun notReadyAlert() {
        reward?.notReadyAlert()
    }

    /**
     * 広告を再生するボタンが画面に表示された事をアプリから通知
     */
    fun showAdPlayButton() {
        reward?.showAdPlayButton()
    }

    /**
     * ユーザーが"CMを観る"ボタンを押したとき事をアプリから通知
     */
    fun clickAdPlayButton() {
        reward?.clickAdPlayButton()
    }

    private val createListener: AdfurikunMovieRewardListener?
        get() {
            listener ?: run {
                listener = object : AdfurikunMovieRewardListener {
                    override fun onPrepareSuccess(isManualMode: Boolean) {
                        channel.invokeMethod(Constants.ON_PREPARE_SUCCESS, AdfurikunCommons.getArgs(number = number, appId = appId))
                    }

                    override fun onPrepareFailure(error: AdfurikunMovieError?) {
                        channel.invokeMethod(Constants.ON_PREPARE_FAILURE, AdfurikunCommons.getArgs(number = number, appId = appId, errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                    }

                    override fun onStartPlaying(data: MovieRewardData) {
                        channel.invokeMethod(Constants.ON_START_PLAYING, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: ""))
                    }

                    override fun onFinishedPlaying(data: MovieRewardData) {
                        channel.invokeMethod(Constants.ON_FINISHED_PLAYING, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: ""))
                    }

                    override fun onFailedPlaying(data: MovieRewardData, error: AdfurikunMovieError?) {
                        channel.invokeMethod(Constants.ON_FAILED_PLAYING, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: "", adNetworkErrorList = error?.adNetworkErrorList))
                    }

                    override fun onAdClose(data: MovieRewardData) {
                        channel.invokeMethod(Constants.ON_AD_CLOSE, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: ""))
                    }
                }
            }
            return listener
        }
}