import 'dart:io';
import 'package:flutter/services.dart';
import 'constants.dart';
import 'ad_base.dart';
import 'adfurikun_plugin.dart';
import 'ad_network_error.dart';

enum AdfurikunInterstitialCallback {
  onPrepareSuccess,
  onPrepareFailure,
  onStartPlaying,
  onFinishedPlaying,
  onAdClose,
  onFailedPlaying
}

typedef AdfurikunInterstitialListener(
    AdfurikunInterstitialCallback? callback,
    String? appId,
    String? adNetworkKey,
    int? errorCode,
    List<AdNetworkError>? adNetworkErrorList);

class AdfurikunInterstitial extends AdfurikunAdBase {
  AdfurikunInterstitialListener? listener;

  Map<String, AdfurikunInterstitialCallback> _callbackMap = {
    CALLBACK_ON_PREPARE_SUCCESS: AdfurikunInterstitialCallback.onPrepareSuccess,
    CALLBACK_ON_PREPARE_FAILURE: AdfurikunInterstitialCallback.onPrepareFailure,
    CALLBACK_ON_START_PLAYING: AdfurikunInterstitialCallback.onStartPlaying,
    CALLBACK_ON_FINISHED_PLAYING: AdfurikunInterstitialCallback.onFinishedPlaying,
    CALLBACK_ON_AD_CLOSE: AdfurikunInterstitialCallback.onAdClose,
    CALLBACK_ON_FAILED_PLAYING: AdfurikunInterstitialCallback.onFailedPlaying
  };

  AdfurikunInterstitial(String appId) : super(appId, AD_TYPE_INTERSTITIAL) {
    AdfurikunPlugin.invoke(method(METHOD_INIT), argsWithPluginVersion());
  }

  void setTrackingId(Map<String, String> trackingId) {
    Map<String, Object> map = args();
    map[KEY_TRACKING_ID] = trackingId;
    AdfurikunPlugin.invoke(method(METHOD_SET_TRACKING_ID), map);
  }

  void load() {
    AdfurikunPlugin.invoke(method(METHOD_LOAD), args());
  }

  void loadWithTimeout(double timeout) {
    Map<String, Object> map = args();
    map[KEY_TIMEOUT] = timeout;
    AdfurikunPlugin.invoke(method(METHOD_LOAD), map);
  }

  void play() {
    AdfurikunPlugin.invoke(method(METHOD_PLAY), args());
  }

  void playWithCustomParams(Map<String, String> customParams) {
    Map<String, Object> map = args();
    map[KEY_CUSTOM_PARAMS] = customParams;
    AdfurikunPlugin.invoke(method(METHOD_PLAY), map);
  }

  Future<bool> isPrepared() async {
    return await AdfurikunPlugin.invokeReturnBool(method(METHOD_IS_PREPARED), args());
  }

  void rewardCompleted(bool result) {
    if (Platform.isIOS) {
      Map<String, Object> map = args();
      map[KEY_BOOL] = result;
      AdfurikunPlugin.invoke(method(METHOD_REWARD_COMPLETED_REWARD), map);
    }
  }

  void notReadyAlert() {
    if (Platform.isIOS) {
      AdfurikunPlugin.invoke(method(METHOD_NOT_READY_ALERT_REWARD), args());
    }
  }

  void showAdPlayButton() {
    if (Platform.isIOS) {
      AdfurikunPlugin.invoke(method(METHOD_SHOW_AD_PLAY_BUTTON_REWARD), args());
    }
  }

  void clickAdPlayButton() {
    if (Platform.isIOS) {
      AdfurikunPlugin.invoke(method(METHOD_CLICK_AD_PLAY_BUTTON_REWARD), args());
    }
  }

  @override
  void destroy() {
    super.destroy();
    AdfurikunPlugin.invoke(method(METHOD_DESTROY), args());
  }

  @override
  void notifyListener(MethodCall call) {
    if (listener != null) {
      AdfurikunInterstitialCallback? callback = _callbackMap[call.method];
      String? appId = call.arguments[KEY_APP_ID];
      String? adNetworkKey = call.arguments[KEY_AD_NETWORK_KEY];
      int? errorCode = call.arguments[KEY_ERROR_CODE];
      List<AdNetworkError>? adNetworkErrorList = convertAdNetworkErrorList(call.arguments[KEY_AD_NETWORK_ERROR_LIST]);
      listener!(callback, appId, adNetworkKey, errorCode, adNetworkErrorList);
    } else {
      print('AdfurikunInterstitialListener is null');
    }
  }
}
